// -------------------------------------------------------------
// 3d code by Georgy Moshkin (Delphi)
// licence: public domain
// -------------------------------------------------------------
unit mynftloader;

interface

uses mymeshes,myvectors,myskeletons,mypolygons,mytextures,
     sysutils,mystrtool;

procedure loadNFT(fname:string;var meshes:TMeshes; var anim:TSkeletonAnimations; var textures:TTextures);

procedure loadVERTS(var f:text; var restverts:TVectors; var vsparents:TVertsParents;count:integer);
procedure loadFACES(var f:text; var polys:TPolygons; var textures:TTextures; count:integer);
procedure loadParents(var f:text; var vparents:TVertParents; count:integer);
procedure loadSKELETON(var f:text; var skeleton:TSkeleton; count:integer);
procedure loadFRAME(var f:text; var reference:TAnimFrame; count:integer);
procedure loadFRAMES(var f:text; var frames:TAnimFrames; count1,count2:integer);

implementation

function readlnTrim(var f:text):string;
var s:string;
begin
     readln(f,s);
     s:=trim( s );
     s:=upperCase(s);
     result:=s;
end;

function readlnParam(var f:text;param:string):string;
var s:string;
begin
 repeat
  s:=readlnTrim(f);
 until (getword(s,1)=param);
result:=s;
end;


procedure loadNFT(fname:string;var meshes:TMeshes; var anim:TSkeletonAnimations; var textures:TTextures);
var f:text;
    s,s1:string;
    i:integer;

    curMesh:integer;
begin
   DecimalSeparator:='.';
   curMesh:=-1;

   assignFile(f,fname);
   reset(f);

   repeat
     s:=readlnTrim(f);
     s1:=getword(s,1);

     if s1='MESH' then begin
                        setlength(meshes,length(meshes)+1);
                        inc(curMesh);
                       end;

     if s1='VERTS' then loadVERTS(f,
                                  meshes[curMesh].restverts,
                                  meshes[curMesh].vertsParents,
                                  strToInt(getword(s,2)));

     if s1='FACES' then loadFACES(f,
                                  meshes[curMesh].polygons,
                                  textures,
                                  strToInt(getword(s,2)));

     if s1='NODES' then loadSKELETON(f,
                                     anim.skeleton,
                                     strToInt(getword(s,2)));

     if s1='REFERENCE' then loadFRAME(f,
                                      anim.reference,
                                      length(anim.skeleton));

     if s1='ACTION' then
                     begin
                      i:=length(anim.actions);
                      setlength(anim.actions,
                                i+1);
                      anim.actions[i].name:=getWord(s,2);
                      anim.actions[i].fps:=StrToInt(
                                            getWord( readlnParam(f,'FPS'), 2)
                                                   );


                      loadFRAMES(f,
                                 anim.actions[i].animFrames,
                                 StrToInt(getWord( readlnParam(f,'FRAMES'),2)),
                                 length(anim.skeleton));


                     end;

   until eof(f);



end;

procedure loadVERTS(var f:text; var restverts:TVectors; var vsparents:TVertsParents;count:integer);
var oldCount,i:integer;
    s:string;

begin
 oldCount:=length(restverts);
 setlength(restverts,  oldCount + count);
 setlength(vsparents, oldCount + count);

 i:=0;
 repeat
  s:=readlnParam(f,'VERTXYZ');
  restverts[oldCount+i].x:=StrToFloat(getword(s,2));
  restverts[oldCount+i].y:=StrToFloat(getword(s,3));
  restverts[oldCount+i].z:=StrToFloat(getword(s,4));

  s:=readlnParam(f,'PARENTS');
  loadParents(f,
              vsparents[oldCount+i],
             StrToInt(getword(s,2)));
  inc(i)
 until i=count;

end;

procedure loadFACES(var f:text; var polys:TPolygons; var textures:TTextures; count:integer);
var oldCount,i:integer;
    s:string;
begin
 repeat
 i:=1;
 until i=1;
 oldCount:=length(polys);
 setlength(polys,  oldCount + count);

 i:=0;
 repeat
  s:=readlnParam(f,'FACE');
  s:=readlnParam(f,'TEXTURE');
  polys[oldCount+i].textureName:=getword(s,2);

  s:=readlnParam(f,'ID1');
  polys[oldCount+i].vertIndex[0]:=StrToInt(getword(s,2));

  s:=readlnParam(f,'ID2');
  polys[oldCount+i].vertIndex[1]:=StrToInt(getword(s,2));

  s:=readlnParam(f,'ID3');
  polys[oldCount+i].vertIndex[2]:=StrToInt(getword(s,2));

  s:=readlnParam(f,'NRM1');
  polys[oldCount+i].normal[0].x:=StrToFloat(getword(s,2));
  polys[oldCount+i].normal[0].y:=StrToFloat(getword(s,3));
  polys[oldCount+i].normal[0].z:=StrToFloat(getword(s,4));

  s:=readlnParam(f,'NRM2');
  polys[oldCount+i].normal[1].x:=StrToFloat(getword(s,2));
  polys[oldCount+i].normal[1].y:=StrToFloat(getword(s,3));
  polys[oldCount+i].normal[1].z:=StrToFloat(getword(s,4));

  s:=readlnParam(f,'NRM3');
  polys[oldCount+i].normal[2].x:=StrToFloat(getword(s,2));
  polys[oldCount+i].normal[2].y:=StrToFloat(getword(s,3));
  polys[oldCount+i].normal[2].z:=StrToFloat(getword(s,4));

  if not (polys[oldCount+i].textureName='NOTEXTURE') then
   begin

    polys[oldCount+i].textureId:=loadMyTexture(textures,polys[oldCount+i].textureName);

    s:=readlnParam(f,'UV1');
    polys[oldCount+i].texcoord[0].x:=StrToFloat(getword(s,2));
    polys[oldCount+i].texcoord[0].y:=StrToFloat(getword(s,3));

    s:=readlnParam(f,'UV2');
    polys[oldCount+i].texcoord[1].x:=StrToFloat(getword(s,2));
    polys[oldCount+i].texcoord[1].y:=StrToFloat(getword(s,3));

    s:=readlnParam(f,'UV3');
    polys[oldCount+i].texcoord[2].x:=StrToFloat(getword(s,2));
    polys[oldCount+i].texcoord[2].y:=StrToFloat(getword(s,3));
   end;

  inc(i);
 until i=count;
 repeat
 i:=1;
 until i=1;
end;

procedure loadParents(var f:text; var vparents:TVertParents; count:integer);
var i:integer;
    s:string;
begin
 setLength(vparents,count);
 for i:=0 to count-1 do
  begin
   s:=readlnTrim(f);
   vparents[i].name:=getWord(s,2);
   vparents[i].weight:=StrToFloat(getWord(s,3));
  end;
end;

procedure loadSKELETON(var f:text; var skeleton:TSkeleton; count:integer);
var i:integer;
    s:string;
begin

 setlength(skeleton,count);
 for i:=0 to count-1 do
  begin
   s:=readlnTrim(f);
   skeleton[i].name:=getWord(s,2);
   skeleton[i].parent:=strToInt(getWord(s,3));
  end;

end;

procedure loadFRAME(var f:text; var reference:TAnimFrame; count:integer);
var i:integer;
    s:string;
begin
 setlength(reference,count);
 for i:=0 to count-1 do
  begin
   s:=readlnTrim(f);
   reference[i].trans.x:=StrToFloat(getWord(s,2));
   reference[i].trans.y:=StrToFloat(getWord(s,3));
   reference[i].trans.z:=StrToFloat(getWord(s,4));
   reference[i].rot.x:=StrToFloat(getWord(s,5));
   reference[i].rot.y:=StrToFloat(getWord(s,6));
   reference[i].rot.z:=StrToFloat(getWord(s,7));
   if getWord(s,8)='YES' then reference[i].YESno:=true
    else reference[i].YESno:=false;

  end;
end;

procedure loadFRAMES(var f:text; var frames:TAnimFrames; count1,count2:integer);
var i:integer;
begin
setlength(frames,count1{check!!!});
for i:=0 to count1-1 do {TO DO: check relations with exporting script}
 begin
  {:=}readlnParam(f,'FRAME');
  loadFRAME(f,frames[i],count2);
 end;


end;


end.
