// -------------------------------------------------------------
// 3d code by Georgy Moshkin (Delphi)
// licence: public domain
// -------------------------------------------------------------
unit myskeletons;

interface

uses myvectors,mymatrix,myquat,math;

type TBonePos=record
                 trans  : TVector;
                 rot    : TVector;
                 absmat : TMatrix;
                 relmat : TMatrix;

                 point1  : TVector;
                 
                 quat   : TQuat;

                 YESno  : boolean;
                end;

type TSkeletonNode=record
                      name       : string;
                      parent     : integer;
                     end;

type TSkeleton=array of TSkeletonNode;

type TAnimFrame=array of TBonePos;

type TAnimFrames=array of TAnimFrame;

type TAction=record
              name       : string;
              fps        : single;
              animFrames : TAnimFrames;
             end;

type TActions=array of TAction;

type TSkeletonState=record
                     prevFrame:integer;
                     nextFrame:integer;
                     prevAction:integer;
                     nextAction:integer;
                     skelTime:single;
                     currentPose: TAnimFrame;
                    end;

type TBoneWeight=record
                  boneIdx: integer;
                  name   : string;
                  weight : single;
                 end;

type TVertParents=array of TBoneWeight;

type TVertsParents=array of TVertParents;


type TSkeletonAnimations=record
                          skeleton : TSkeleton;
                          actions  : TActions;
                          reference  : TAnimFrame;
                        end;

procedure UpdateQuats(var Pose:TAnimFrame);
procedure Anim(actions:TActions;var state:TSkeletonState;actionNum:integer;BlenderElapsed:single);
procedure Transform(var verts:TVectors; restverts:TVectors;vparents:TVertsParents;pose,restpose:TAnimFrame;skel:TSkeleton);
function IndexFromName(skel:TSkeleton;name:string):integer;
procedure BuildSkeleton(restverts:TVectors;var transverts:TVectors;vparents:TVertsParents;var pose:TAnimFrame;skel:TSkeleton);
procedure BuildFrame(var Pose:TAnimFrame; skel:TSkeleton);

implementation

procedure UpdateQuats(var Pose:TAnimFrame);
var i:integer;
begin
     // TODO? NEED FIX FOR FIRST FRAMES
   for i:=0 to length(Pose)-1 do
    begin
     Pose[i].quat:=QuatFromAngles(Pose[i].rot);
     Pose[i].relmat:=QuatMat(Pose[i].quat);
    end;

end;

procedure Anim(actions:TActions;var state:TSkeletonState;actionNum:integer;BlenderElapsed:single);
var delta:single;
var i,k:integer;
    db:TVector;
begin

with state do
 begin

 // some hack
 if length(currentPose)=0 then setlength(currentPose,length(Actions[prevAction].animFrames[prevFrame]));
 

  if prevAction<>ActionNum then
   begin
    nextAction:=ActionNum;
    nextFrame:=0;

    /// ????ALSO UPDATE HERE TO REMOVE ACTION SWITCH BONE BUG????
     updateQuats(Actions[prevAction].animFrames[prevFrame]);
     updateQuats(Actions[nextAction].animFrames[nextFrame]);
   end;

  if prevAction=-1 then prevAction:=ActionNum;
  if nextAction=-1 then nextAction:=ActionNum;

  delta:= BlenderElapsed * Actions[nextAction].fps / 1000;

  if skelTime=0 then  // TODO? UPDATE FOR FIRST FRAMES CORRECT????
   begin
    updateQuats(Actions[prevAction].animFrames[prevFrame]);
    updateQuats(Actions[nextAction].animFrames[nextFrame]);
   end;

   

  skelTime:=skelTime + delta;

  if skelTime>1 then
   begin
    skelTime:=0;
    prevAction:=nextAction;
    prevFrame:=nextFrame;
    nextFrame:=nextFrame+1; //aDelta;

    if nextFrame>length(Actions[nextAction].animFrames){-1}-2{!!!} then
     begin
      prevAction:=nextAction; // HACK
      nextFrame:=0; // ??? 0 and last frames equal?? in old versions 0 - reference.
     end;


   end;


 for i:=0 to length(currentPose)-1 do
  begin

    currentpose[i].trans:=vectorLerp(Actions[prevAction].animFrames[prevFrame][i].trans,
                                   Actions[nextAction].animFrames[nextFrame][i].trans,
                                   skelTime);

    with Actions[prevAction].animFrames[prevFrame][i] do
    begin
     currentpose[i].quat:=QuatLerp(Actions[prevAction].animFrames[prevFrame][i].quat,
                                   Actions[NextAction].animFrames[NextFrame][i].quat,
                                   skelTime);
      currentpose[i].relmat:=QuatMat(currentpose[i].quat);
    end;

  end;


 end;
end;

procedure Transform(var verts:TVectors; restverts:TVectors;vparents:TVertsParents;pose,restpose:TAnimFrame;skel:TSkeleton);
var i,k:integer;
    zzz,fff:TVector;
    mmm:TMatrix;
    w:single;

begin

for i:=0 to length(verts)-1 do
 begin
  fff.x:=0;
  fff.y:=0;
  fff.z:=0;
 for k:=0 to length(vparents[i])-1 do
  begin
   zzz:=restverts[i];

   mmm:=restpose[vparents[i][k].boneIdx].absmat;

   InvApplyMatrix(zzz,mmm);

   mmm:=pose[vparents[i][k].boneIdx].absmat;

   ApplyMatrix(zzz,mmm);

   w:=vparents[i][k].weight;

   fff.x:=fff.x+zzz.x*w;
   fff.y:=fff.y+zzz.y*w;
   fff.z:=fff.z+zzz.z*w;
  end;
  verts[i]:=fff;
 end;



end;



function IndexFromName(skel:TSkeleton;name:string):integer;
var i:integer;
    found:boolean;
begin
found:=false;
 for i:=0 to length(skel)-1 do
  if skel[i].name=name then
   begin
    result:=i;
    found:=true;
   end;

if not found then exit;   
end;

procedure BuildSkeleton(restverts:TVectors;var transverts:TVectors;vparents:TVertsParents;var pose:TAnimFrame;skel:TSkeleton);
var i,k:integer;
    zzz,fff:TVector;
    mmm:TMatrix;
    w:single;
begin



updateQuats(pose);
BuildFrame(pose,skel);

setlength(transverts,length(restverts));

for i:=0 to length(restverts)-1 do
  transverts[i]:=restverts[i];

for i:=0 to length(restverts)-1 do
 for k:=0 to length(vparents[i])-1 do
   vparents[i][k].boneIdx:=IndexFromName(skel,vparents[i][k].name);

end;


procedure BuildFrame(var Pose:TAnimFrame; skel:TSkeleton);
var i:integer;
begin

for i:=0 to length(pose)-1 do
 begin
  pose[i].point1.x:=0;
  pose[i].point1.y:=0;
  pose[i].point1.z:=0;

(* TODO: FIRST FRAMES: USE QUATS OR THIS???
  pose[i].relmat:=rotMatX(pose[i].rot.x);
  pose[i].relmat:=ConcatMatrix(rotMatY(pose[i].rot.y),pose[i].relmat);
  pose[i].relmat:=ConcatMatrix(rotMatZ(pose[i].rot.z),pose[i].relmat);
*)

  pose[i].relmat[0,3]:=pose[i].trans.x;
  pose[i].relmat[1,3]:=pose[i].trans.y;
  pose[i].relmat[2,3]:=pose[i].trans.z;

       if skel[i].parent<>-1 then
         pose[i].absmat:=ConcatMatrix(pose[skel[i].parent].absmat,
                                          pose[i].relmat)
         else
        pose[i].absmat:=pose[i].relmat;


 end;


   for i:=0 to length(pose)-1 do
   ApplyMatrix(pose[i].point1,
               pose[i].absmat);

end;



end.


